#!/usr/bin/python

# compute the discrete fourier transform of data from file

import sys
import numpy as np
import pylab as p
import matplotlib.pyplot as plt
from gatspy.periodic import LombScargleFast

#Vordefinierung Grid
def freq_grid(t, oversampling=0, nyquist_factor=3):
    T = NX*DT
    N = len(t)
    
    df = 1/DT
    fmax = 0.5 * nyquist_factor * N / T
    N = int(fmax // df)
    return df + df * np.arange(N)

if len(sys.argv) > 1:
        filename=sys.argv[1]
else:
        print("usage: python fouriertransform.py <filename>\n")
        sys.exit(1)

with open(filename) as file:
     floats = list(map(float,file))

#Zeug fuer Frequenzen Feld:

NX = int(len(floats))
t = [0]*NX
DT = 1.0
print("delta_t : %f" % DT)
for i in range(NX):
    t[i] = DT*i
freq = freq_grid(t)

# data
UX = np.array(floats)
# number of data values
NT = int(len(floats))

print("data average   : %f" % (UX.mean()))
UX = UX - UX.mean()

# time step DT = (tmax-tmin)/(NT-1)

# smallest frequency 
DF = 1.0/((NT-1.0)*DT)

#LS Magic:

model = LombScargleFast().fit(t, UX, 0.01)
period, power = model.periodogram_auto()

period=1/period

xmax = period[np.argmax(power)]-1.0
ymax = power[np.argmax(power)]

#Plot Magic:

p.plot(period, power)
#p.axvline(xmax,c='r',linestyle="--")
p.annotate("f = %0.5f Hz" % xmax, (xmax*1.03,ymax*0.95))
p.xlim(0.0001, 0.5)
p.ylim(0.00001,ymax*2)
p.xscale("log")
p.yscale("log")
p.xlabel('$f / Hz$')
p.ylabel('$power$')
p.grid()

file = open(filename+'_LS.dat',"w")  
for i in range(int(NX/4)):
    file.write("%f %f \n" % (1/period[i],power[i]))

print("Max(Freq) : %0.5f Hz (%0.5f) "  % ((xmax), ymax))

p.savefig(filename+'_LS.png')
p.show()
